import React, { useEffect, useState } from 'react';
import PageContainer from '@/components/BasePageContainer';
import { notification, Spin, Tabs } from 'antd';
import {
  getWebModuleTree,
  getWebconfig,
  postEditWebConfig,
} from '@/services/webConfig/api';
import { cancelled } from 'redux-saga/effects';
import styles from './index.less';
import SiteConfig from './components/siteConfigDrawer';
import { appConnector } from '@/containers/App/store';
const { TabPane } = Tabs;

const WebConfigPage = props => {
  const { userMode } = props;
  const [configVisible, setConfigVisible] = useState(false);
  const [loading, setLoading] = useState(false);
  const [webs, setWebs] = useState([]);
  const [curWeb, setCurWeb] = useState(null); // 当前展示的web
  const [configObj, setConfigObj] = useState({}); // 获取当前的web的配置
  const [toEdit, setToEdit] = useState(null); // 编辑展示用的配置
  const [isEdit, setIsEdit] = useState(true);
  const [submitting, setSubmitting] = useState(false);

  const hasIntegerate = () => webs.some(w => w.mode === 'integrate');

  useEffect(() => {
    let canceled = { cancel: false };
    setLoading(true);
    updateModuleTree(userMode || 'super', canceled);
    return () => {
      canceled.cancel = true;
    };
  }, []);

  useEffect(() => {
    let canceled = { cancel: false };
    if (!curWeb) return;
    const title = curWeb.text;
    updateWebconfig(title, canceled);

    // eslint-disable-next-line consistent-return
    return () => {
      canceled.cancel = true;
    };
  }, [curWeb]);

  const updateModuleTree = (userModePrama, canceled = { cancel: false }) => {
    getWebModuleTree(userModePrama).then(res => {
      const websArr = [
        res.data
          .filter(d => d.id === 'Web4IntegrateStation')
          .map(r => r.children),
        res.data.filter(d => d.id === 'Web4SingleStation').map(r => r.children),
      ].flat(2);
      if (!canceled.cancel) {
        setWebs(websArr);
        if (!curWeb) setCurWeb(websArr[0]);
        setLoading(false);
      }
    });
  };

  const updateWebconfig = (webTitle, canceled = { cancel: false }) => {
    setLoading(true);
    return getWebconfig(webTitle)
      .then(res => {
        setLoading(false);
        if (!canceled.cancel) {
          setConfigObj(res);
          if (webTitle === curWeb.text) {
            setToEdit(res);
          }
        }
      })
      .catch(err => {
        setLoading(false);
        // eslint-disable-next-line no-console
        console.error(err);
      });
  };

  const renderTabPane = tabPaneItem => (
    <TabPane key={tabPaneItem.text} tab={tabPaneItem.text}>
      <>
        <span
          className={styles.link}
          onClick={() => {
            setToEdit({ ...configObj });
            setConfigVisible(true);
            setIsEdit(true);
          }}
        >
          查看/编辑网站配置
        </span>
      </>
    </TabPane>
  );

  const onEdit = (targetKey, action) => {
    // console.log(targetKey, action); // action: add|remove
    switch (action) {
      case 'add':
        setIsEdit(false);
        setToEdit({});
        setConfigVisible(true);
        break;
      case 'remove':
        break;
      default:
    }
  };

  const onDrawerClose = form => {
    setConfigVisible(false);
  };

  const handleTabChange = v => {
    setCurWeb(webs.find(w => w.text === v));
  };

  const handleSubmit = values => {
    setSubmitting(true);
    if (isEdit) {
      postEditWebConfig(values)
        .then(res => {
          setSubmitting(false);
          if (res.success) {
            setCurWeb({ ...curWeb, text: values.title });
            notification.success({
              message: '保存成功!',
              duration: 3,
            });
            updateModuleTree(userMode || 'super');
          } else {
            notification.warning({
              message: res.message || '编辑失败！',
              duration: 5,
            });
          }
        })
        .catch(err => {
          notification.success({
            message: '编辑失败！',
            duration: 5,
          });
          // eslint-disable-next-line no-console
          console.error(err);
          setSubmitting(false);
        });
    }
  };

  return (
    <PageContainer>
      <div className={styles.webConfigContainer}>
        <Spin spinning={loading || submitting}>
          <Tabs type="editable-card" onEdit={onEdit} onChange={handleTabChange}>
            {webs.map(renderTabPane)}
          </Tabs>
          <SiteConfig
            isEdit={isEdit}
            visible={configVisible}
            onClose={onDrawerClose}
            config={toEdit}
            onOk={handleSubmit}
            submitting={submitting}
            hasIntegerate={hasIntegerate()}
          />
        </Spin>
      </div>
    </PageContainer>
  );
};

export default appConnector(WebConfigPage);
